'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AuthIdCache = undefined;

var _node = require('parse/node');

var _node2 = _interopRequireDefault(_node);

var _lruCache = require('lru-cache');

var _lruCache2 = _interopRequireDefault(_lruCache);

var _logger = require('../logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function userForAuthId(authIdKey, authIdValue) {
  var q = new _node2.default.Query("_User");
  q.equalTo(authIdKey, authIdValue);
  return q.first({ useMasterKey: true }).then(function (user) {
    if (!user) {
      return _node2.default.Promise.error("No user found for authIdValue");
    }
    return user;
  });
}

class AuthIdCache {

  constructor(timeout = 30 * 24 * 60 * 60 * 1000, maxSize = 10000) {
    this.cache = new _lruCache2.default({
      max: maxSize,
      maxAge: timeout
    });
  }

  getUserId(authIdKey, authIdValue) {
    if (!authIdValue) {
      return _node2.default.Promise.error(`Empty authId key: [${authIdKey}]`);
    }
    const userId = this.cache.get(authIdValue);
    if (userId) {
      _logger2.default.verbose('Fetch userId %s of authIdValue %s from Cache', userId, authIdValue);
      return _node2.default.Promise.as(userId);
    }
    return userForAuthId(authIdKey, authIdValue).then(user => {
      _logger2.default.verbose('Fetch userId %s of authIdValue %s from Parse', user.id, authIdValue);
      const userId = user.id;
      this.cache.set(authIdValue, userId);
      return _node2.default.Promise.as(userId);
    }, error => {
      _logger2.default.error('Can not fetch userId for authIdValue %j, error %j', authIdValue, error);
      return _node2.default.Promise.error(error);
    });
  }
}

exports.AuthIdCache = AuthIdCache;